--- src/libvlc-module.c	Fri May 11 14:06:25 2007
+++ src/libvlc-module.c	Fri May 11 14:06:14 2007
@@ -553,6 +553,13 @@
     "VLC can display messages on the video. This is called OSD (On Screen " \
     "Display).")
 
+#define REPOS_TEXT N_("Reposition sub-pictures")
+#define REPOS_LONGTEXT N_( \
+        "Don't allow overlapping sub-pictures.")
+
+#define REPOS_X_TEXT N_("Sub-picture space")
+#define REPOS_X_LONGTEXT N_("Spacing distance between the sub-pictures.")
+
 #define TEXTRENDERER_TEXT N_("Text rendering module")
 #define TEXTRENDERER_LONGTEXT N_( \
     "VLC normally uses Freetype for rendering, but this allows you to use svg for instance.")
@@ -1388,6 +1395,11 @@
     add_bool( "osd", 1, NULL, OSD_TEXT, OSD_LONGTEXT, VLC_FALSE );
     add_module( "text-renderer", "text renderer", NULL, NULL, TEXTRENDERER_TEXT,
                 TEXTRENDERER_LONGTEXT, VLC_TRUE );
+
+    add_bool( "reposition-subs", 0, NULL, REPOS_TEXT, REPOS_LONGTEXT,
+            VLC_TRUE );
+    add_integer( "reposition-spacing", 10, NULL, REPOS_X_TEXT,
+            REPOS_X_LONGTEXT, VLC_TRUE );
 
     set_section( N_("Subtitles") , NULL );
     add_file( "sub-file", NULL, NULL, SUB_FILE_TEXT,
--- src/video_output/vout_subpictures.c	Tue May  8 16:11:48 2007
+++ src/video_output/vout_subpictures.c	Thu May 10 19:02:09 2007
@@ -505,6 +505,70 @@
                             subpicture_t *p_subpic,
                             int i_scale_width_orig, int i_scale_height_orig )
 {
+    int i_bottom_bottom = p_fmt->i_height;
+    int i_bottom_top = 0;
+    int i_top_bottom = p_fmt->i_height;
+    int i_top_top = 0;
+    vlc_bool_t b_reposition_subs =
+        var_CreateGetInteger( p_spu, "reposition-subs" );
+    int i_reposition_spacing = 0;
+
+    if( b_reposition_subs )
+    {
+        subpicture_t *p_subpic_tmp;
+
+        i_reposition_spacing =
+            var_CreateGetInteger( p_spu, "reposition-spacing" );
+
+        for( p_subpic_tmp = p_subpic;
+                p_subpic_tmp != NULL &&
+                p_subpic_tmp->i_status != FREE_SUBPICTURE;
+                p_subpic_tmp = p_subpic_tmp->p_next )
+        {
+            subpicture_region_t *p_region = p_subpic_tmp->p_region;
+            int i_scale_height;
+
+            i_scale_height = i_scale_height_orig;
+
+            if( p_subpic->i_original_picture_height > 0 &&
+                    p_subpic->i_original_picture_width  > 0 )
+            {
+                i_scale_height = i_scale_height * p_fmt->i_height /
+                    p_subpic->i_original_picture_height;
+            }
+            else if( p_subpic->i_original_picture_height > 0 )
+            {
+                i_scale_height = i_scale_height * p_fmt->i_height /
+                    p_subpic->i_original_picture_height;
+            }
+
+            for( ; p_region; p_region = p_region->p_next )
+            {
+                if( p_region->fmt.i_chroma == VLC_FOURCC('T','E','X','T') )
+                    continue;
+
+                if( p_subpic_tmp->i_flags & SUBPICTURE_ALIGN_BOTTOM )
+                {
+                    int i_tmp_height = i_scale_height *
+                        p_region->fmt.i_height / 1000;
+                    i_bottom_bottom = __MIN( i_bottom_bottom,
+                            p_subpic_tmp->i_y );
+                    i_bottom_top = __MAX( i_bottom_top,
+                            p_subpic_tmp->i_y + i_tmp_height );
+                }
+                else if( p_subpic_tmp->i_flags & SUBPICTURE_ALIGN_TOP )
+                {
+                    int i_tmp_height = i_scale_height *
+                        p_region->fmt.i_height / 1000;
+                    i_top_bottom = __MIN( i_top_bottom,
+                            p_subpic_tmp->i_y );
+                    i_top_top = __MAX( i_top_top,
+                            p_subpic_tmp->i_y + i_tmp_height );
+                }
+            }
+        }
+    }
+
     /* Get lock */
     vlc_mutex_lock( &p_spu->subpicture_lock );
 
@@ -652,8 +716,8 @@
         while( p_region && p_spu->p_blend && p_spu->p_blend->pf_video_blend )
         {
             int i_fade_alpha = 255;
-            int i_x_offset = p_region->i_x + i_subpic_x;
-            int i_y_offset = p_region->i_y + p_subpic->i_y;
+            int i_x_offset;
+            int i_y_offset;
 
             if( p_region->fmt.i_chroma == VLC_FOURCC('T','E','X','T') )
             {
@@ -672,6 +736,69 @@
                                                        p_region, p_region );
                     }
                 }
+
+                if( b_reposition_subs )
+                {
+                    int i_border_dist = i_reposition_spacing;
+
+                    if( i_border_dist <= 0 ) i_border_dist = p_subpic->i_y;
+
+                    if( p_subpic->i_flags & SUBPICTURE_ALIGN_BOTTOM )
+                    {
+                        int i_tmp_height = i_scale_height *
+                            p_region->fmt.i_height / 1000;
+
+                        if( i_bottom_top )
+                        {
+                            if( 2 * i_border_dist +
+                                    i_tmp_height > i_bottom_bottom )
+                            {
+                                p_subpic->i_y = i_bottom_top + i_border_dist;
+                            }
+                            else
+                            {
+                                p_subpic->i_y = i_bottom_bottom -
+                                    i_border_dist - i_tmp_height;
+                            }
+                        }
+                        else
+                        {
+                            p_subpic->i_y = i_border_dist;
+                        }
+
+                        i_bottom_bottom = __MIN( i_bottom_bottom,
+                                p_subpic->i_y );
+                        i_bottom_top = __MAX( i_bottom_top, p_subpic->i_y +
+                                i_tmp_height );
+                    }
+                    else if( p_subpic->i_flags & SUBPICTURE_ALIGN_TOP )
+                    {
+                        int i_tmp_height = i_scale_height *
+                            p_region->fmt.i_height / 1000;
+
+                        if( i_top_top )
+                        {
+                            if( 2 * i_border_dist +
+                                    i_tmp_height > i_top_bottom )
+                            {
+                                p_subpic->i_y = i_top_top + i_border_dist;
+                            }
+                            else
+                            {
+                                p_subpic->i_y = i_top_bottom -
+                                    i_border_dist - i_tmp_height;
+                            }
+                        }
+                        else
+                        {
+                            p_subpic->i_y = i_border_dist;
+                        }
+
+                        i_top_bottom = __MIN( i_top_bottom, p_subpic->i_y );
+                        i_top_top = __MAX( i_top_top, p_subpic->i_y +
+                                i_tmp_height );
+                    }
+                }
             }
 
             /* Force palette if requested */
@@ -748,7 +875,11 @@
                 i_y_offset = p_fmt->i_height - p_region->fmt.i_height -
                     p_subpic->i_y;
             }
-            else if ( !(p_subpic->i_flags & SUBPICTURE_ALIGN_TOP) )
+            else if ( p_subpic->i_flags & SUBPICTURE_ALIGN_TOP )
+            {
+                i_y_offset = p_region->i_y + p_subpic->i_y;
+            }
+            else
             {
                 i_y_offset = p_fmt->i_height / 2 - p_region->fmt.i_height / 2;
             }
@@ -758,7 +889,11 @@
                 i_x_offset = p_fmt->i_width - p_region->fmt.i_width -
                     i_subpic_x;
             }
-            else if ( !(p_subpic->i_flags & SUBPICTURE_ALIGN_LEFT) )
+            else if ( p_subpic->i_flags & SUBPICTURE_ALIGN_LEFT )
+            {
+                i_x_offset = p_region->i_x + i_subpic_x;
+            }
+            else
             {
                 i_x_offset = p_fmt->i_width / 2 - p_region->fmt.i_width / 2;
             }
